/*
   This file is part of msplot, a visualisation tool for Massif output

   Copyright (C) 2007-2008 Martijn Versteegh


   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307, USA.

   The GNU General Public License is contained in the file COPYING.
*/
#include "plotsession.h"

void const *SessionPlot::GetRootNode()
{
	if (!session)
		return NULL;

	return session->GetSnapShot(snapShot)->GetAllocTree();
}


unsigned int SessionPlot::GetNumChildren(void const *handle)
{
	TreeNode const *t = (TreeNode const *)handle;

	if (t->GetNumChildren() == 1)
		return 0;

	return t->GetNumChildren();
}


void const *SessionPlot::GetChild(void const *handle, int child)
{
	TreeNode const *t = (TreeNode const *)handle;
	return t->GetChild(child);
}

void const *SessionPlot::GetParent(void const *handle)
{
	TreeNode const *t = (TreeNode const *)handle;
	return t->GetParent();
}

unsigned int SessionPlot::GetValue(void const *handle)
{
	TreeNode const *t = (TreeNode const *)handle;
	return t->GetSize();
}


unsigned int SessionPlot::GetId(void const *handle)
{
	TreeNode const *t = (TreeNode const *)handle;
	return t->GetId();
}

char const *SessionPlot::GetName(void const *handle)
{
	TreeNode const *t = (TreeNode const *)handle;
	return t->GetName();

}

void SessionPlot::OnMouseEvt(wxMouseEvent &evt, void const *handle)
{
	if (!handle)
		return;

	TreeNode const *t = (TreeNode const *)handle;

	if (evt.LeftDown())
	{
		numDisplayedIds++;
		if (numDisplayedIds > MAXNUMDISPLAYEDIDS)
			numDisplayedIds = MAXNUMDISPLAYEDIDS;

		for (int i = numDisplayedIds; i > 0; i--)
		{
			displayedIds[i] = displayedIds[i - 1];
		}

		displayedIds[0] = handle;
		DoRefresh();
	}
	else if (evt.LeftDClick())
	{
		unsigned int old = rootNodeId;
		rootNodeId = t->GetId();

		if (old == rootNodeId)
		{
			TreeNode const *p = t->GetParent();

			rootNodeId = p ? p->GetId() : 0;
		}


		DoRefresh();
	}

}

void  SessionPlot::CustomDrawPost(wxDC *bitmap)
{
	int h = 0;
	bitmap->GetTextExtent(wxT("A"), NULL, &h);
	wxString txt;

	h+=2;

	for (int i = 0; i < numDisplayedIds; i++)
	{
		TreeNode const *t = (TreeNode const *)displayedIds[i];

		txt = wxString::FromAscii( t->GetName());

		bitmap->DrawText(txt,1, 30 + h * i);

		wxPoint s;

		bitmap->GetTextExtent(txt, &(s.x), &(s.y));

		s.x += 2;
		s.y /=2 ;
		s.y += (30 + h * i);

		wxPoint e = GetItemCentre(t->GetId());

		bitmap->DrawLine(s, e);

	}
}


